@@ -1,3 +1,28 @@
+# Copyright (C) 2019- Centre of Biological Engineering,
+#     University of Minho, Portugal
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
+""" 
+##############################################################################
+Compartementalized community Model inspired in REFRAMED.
+Can build community models from models loaded from any toolbox
+for which there is a Simulator implementation.
+
+Author: Vitor Pereira
+##############################################################################
+"""
 from mewpy.simulation import get_simulator
 from mewpy.simulation.simulation import Simulator
 from mewpy.util.parsing import build_tree, Boolean
