@@ -1,4 +1,24 @@
-""" Inspyred Operators
+# Copyright (C) 2019- Centre of Biological Engineering,
+#     University of Minho, Portugal
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+"""
+##############################################################################
+Genetic operators for inspyred
+
+Authors: Vitor Pereira
+##############################################################################
 """
 
 import copy
@@ -183,10 +203,12 @@ def grow_mutation_KO(random, candidate, args):
 
     mutant = copy.copy(candidate)
     if len(mutant) < maxSize:
-        newElem = random.randint(bounder.lower_bound, bounder.upper_bound)
-        while newElem in mutant:
-            newElem = random.randint(bounder.lower_bound, bounder.upper_bound)
-        mutant.add(newElem)
+        idx = random.randint(bounder.lower_bound, bounder.upper_bound)
+        while idx in mutant:
+            idx = idx + 1
+            if idx > bounder.upper_bound:
+                idx = bounder.lower_bound
+        mutant.add(idx)
     return mutant
 
 
@@ -221,10 +243,11 @@ def grow_mutation_OU(random, candidate, args):
     mutant = copy.copy(candidate)
     if len(mutant) < maxSize:
         idx = random.randint(bounder.lower_bound[0], bounder.upper_bound[0])
-        idxs = [a for (a, b) in mutant]
+        idxs = [a for (a, _) in mutant]
         while idx in idxs:
-            idx = random.randint(
-                bounder.lower_bound[0], bounder.upper_bound[0])
+            idx = idx + 1
+            if idx > bounder.upper_bound[0]:
+                idx = bounder.lower_bound[0]
         lv = random.randint(bounder.lower_bound[1], bounder.upper_bound[1])
         mutant.add((idx, lv))
     return mutant
@@ -247,15 +270,18 @@ def single_mutation_KO(random, candidate, args):
 
     bounder = args["_ec"].bounder
     mutRate = args.setdefault("mutation_rate", 0.1)
-    if random.random() > mutRate:
+    n = bounder.upper_bound - bounder.lower_bound + 1
+    if random.random() > mutRate or len(candidate) >= n:
         return candidate
     mutant = copy.copy(candidate)
     index = random.randint(0, len(mutant) - 1) if len(mutant) > 1 else 0
-    newElem = random.randint(bounder.lower_bound, bounder.upper_bound)
-    while newElem in mutant:
-        newElem = random.randint(bounder.lower_bound, bounder.upper_bound)
+    idx = random.randint(bounder.lower_bound, bounder.upper_bound)
+    while idx in mutant:
+        idx = idx + 1
+        if idx > bounder.upper_bound:
+            idx = bounder.lower_bound
     mutantL = list(mutant)
-    mutantL[index] = newElem
+    mutantL[index] = idx
     mutant = set(mutantL)
     return mutant
 
@@ -282,10 +308,8 @@ def single_mutation_OU(random, candidate, args):
 
     import random
     id = random.randint(1, 1000)
-    if minSize == maxSize and minSize == bounder.upper_bound[0]+1:
-        return candidate
-
-    if random.random() > mutRate:
+    n = bounder.upper_bound[0] - bounder.lower_bound[0] + 1
+    if random.random() > mutRate or len(candidate) >= n:
         return candidate
 
     mutant = copy.copy(candidate)
@@ -299,8 +323,9 @@ def single_mutation_OU(random, candidate, args):
     if random.random() > 0.5:
         idx = random.randint(bounder.lower_bound[0], bounder.upper_bound[0])
         while idx in ml:
-            idx = random.randint(
-                bounder.lower_bound[0], bounder.upper_bound[0])
+            idx = idx + 1
+            if idx > bounder.upper_bound[0]:
+               idx = bounder.lower_bound[0]
         is_mutate_idx = True
     lv = random.randint(bounder.lower_bound[1], bounder.upper_bound[1])
     while not is_mutate_idx and lv == idy:
