@@ -1,5 +1,25 @@
+# Copyright (C) 2019- Centre of Biological Engineering,
+#     University of Minho, Portugal
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 """
-Simulation for COBRApy models
+##############################################################################
+Simulation for COBRApy models. Provide a wrapper with a common interface
+to COBRApy.
+
+Author: Vítor Pereira
+##############################################################################
 """
 import logging
 from collections import OrderedDict
@@ -833,8 +853,34 @@ class GeckoSimulation(Simulation):
         if len(values) == 1:
             m_r = self.metabolite_reaction_lookup()
             r_d = m_r[values[0]]
-            return {k: -1/v for k, v in r_d.items()}
+            return {k: -1/v for k, v in r_d.items() if self.protein_prefix not in k}
         elif len(values) > 1:
             raise ValueError(f"More than one protein match {values}")
         else:
             raise ValueError(f"Protein {protein} not founded.")
+
+    def set_Kcat(self, protein, reaction, kcat):
+        """Alters an enzyme kcat for a given reaction.
+
+        :param protein: The protein identifier
+        :type protein: str
+        :param reaction: The reaction identifier
+        :type reaction: str
+        :param kcat: The kcat value, a real positive value.
+        :type kcat: float
+        """
+        if kcat <= 0:
+            raise ValueError('kcat value needs to be positive.')
+
+        rxn = self.model.reactions.get_by_id(reaction)
+        m = None
+        for met in rxn.metabolites:
+            if protein in met.id:
+                m = met
+                break
+        if met is not None:
+            rxn.metabolites[met]=kcat
+        else:
+            LOGGER.warn(f'Could not identify {protein} ' 
+                        f'protein specie in reaction {reaction}')
+ 
\ No newline at end of file
