@@ -1,4 +1,27 @@
-""" Adapted from Daniel Machado's REFRAMED"""
+# Copyright (C) 2019- Centre of Biological Engineering,
+#     University of Minho, Portugal
+
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+"""
+##############################################################################
+CPLEX solvers interface
+
+Adapted by Vitor Pereira from Daniel Machado's REFRAMED
+https://github.com/cdanielmachado/reframed
+
+##############################################################################
+"""
 from .solver import Solver, VarType, Parameter, default_parameters
 from .solution import Solution, Status
 from cplex import Cplex, infinity, SparsePair
